import functools
import requests

# 数据读取相关
def read_file(file_path: str) -> str:
    """
    读取文件内容
    :param file_path: 文件路径
    :return: 文件内容字符串
    """
    try:
        with open(file_path, 'r', encoding="utf-8") as f:
            return f.read()
    except FileNotFoundError:
        print(f"文件 {file_path} 未找到")
        return ""

## 其他

def status(task:str,date:str,run_only = False):
    """执行任务并记录执行状态, 如果已经执行了则不再执行(每日)

    Args:
        task (str): 任务名称
        date (str): 2019-12-12

    Returns:
        str: '是否成功'
    """
    def outer_packing(func):
        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            response = requests.get(
                f"http://101.201.244.227:8000/action/{task}/{date}/status",timeout=20)
            if response.json().get('status') == '未执行':
                result = func(*args, **kwargs)
            else:
                result = None
            if run_only:
                response = requests.post(
                f"http://101.201.244.227:8000/action/{task}/{date}/execute",timeout=20)
            return result
        return wrapper
    return outer_packing



import re


def extract_type_code(text: str)->str:
    """从文本中提取python代码

    Args:
        text (str): 输入的文本。

    Returns:
        str: 提取出的python文本
    """
    pattern = r'```type([\s\S]*?)```'
    matches = re.findall(pattern, text)
    return matches[0].replace('\n','')



""" 提供对外的方法 提供一些带状态的消息 """

from datetime import datetime
from .action.action import sync_calulate,sync_weight,sync_ready_pool,sync_order_pool
from .action.action import sync_run_pool, sync_note, sync_news, sync_note_night, recycle_tasks

from .nodes import clean_and_update,edit_coder, test_and_study
from .nodes import design, meeting_and_talk
from .nodes import task_failed,task_complete

from .utils import check_action_,check_, TaskInfo

from .action.scripts import run_shortcut, display_dialog, display_dialog_for_end, judge_type
from .utils import create_func,push_task
from reallife import Date, Setting


def morning()->str:
    """清晨动作

    Returns:
        str: 系统消息
    """
    date = Date().date

    try:
        check_(create_func(task='记录灵感',date=date))
        check_(create_func(task='记录体重',date=date))
        check_(create_func(task='跑步',date=date))
        check_(create_func(task='记录吃了什么',date=date))
        check_(create_func(task='洗漱护肤',date=date))
        check_(create_func(task='收拾家与锻炼',date=date))
    except TaskInfo as e:
        return e

    return 'success'

def start_work(debug=True)->str:
    """开工

    Args:
        debug (bool, optional): 是否调试模式. Defaults to True.

    Returns:
        str: 系统消息
    """
    date = Date().date
    try:
        check_(create_func(task='上班打卡',date=date))
        check_action_(sync_ready_pool,debug)
        check_action_(sync_order_pool,debug)
        check_action_(sync_weight,debug)
        check_(create_func(task='调整优先级',date=date))
        check_action_(sync_run_pool,debug)
        # check_action_(sync_news) # 对方改版了,等着用pypeteer去做吧
        check_action_(sync_calulate,debug)
        check_action_(sync_note,debug)
        check_(create_func(task='倒水茶水',date=date))
        # bulid知识库
    except TaskInfo as e:
        return e

    return 'success'

def tasks():
    """任务列表

    Returns:
        str: 系统消息
    """
    date = Date().date
    try:
        check_(create_func(task='写下灵感',date=date))
        check_(create_func(task='跟进阻塞池',date=date))
        task = run_shortcut("获取任务")
        print("task:",task)
        if not task:
            return '无任务'
        task_type = judge_type(task)
        if task_type == "代码与练习":
            edit_coder(task)
        elif task_type == "实验与学习":
            test_and_study(task)
        elif task_type == "整理与优化":
            clean_and_update(task)
        elif task_type == "设计":
            design(task)
        elif task_type == "开会与对齐":
            meeting_and_talk(task)
        else:
            print('新的类别')
            # 开会与对齐
            meeting_and_talk(task)

        task_result = display_dialog_for_end("判断",f"任务是否完成: ",button_text="complete",button_text2="blockage",button_cancel=True)

        if task_result == 'complete':
            # 完成任务
            task_complete(task=task)
        
        elif task_result == 'blockage':
            # 任务阻塞
            #TODO
            pass

        else:
            # 完成未任务
            task_failed(task=task)

        
        # 移除完成的任务
        run_shortcut("移除完成的任务",task)

    except TaskInfo as e:
        return e

    return 'success'


def finish_work():
    """收工动作

    Returns:
        str: 系统信息
    """
    date = Date().date
    try:
        #TODO 知识库整理
        check_action_(recycle_tasks)
        check_action_(sync_note_night)
        check_(create_func(task='检查和收集咨询',date=date))
        check_(create_func(task='检查是否仍在禅模式',date=date))
        check_(create_func(task='检测git提交',date=date))
        check_(create_func(task='下班打卡',date=date))
        check_(create_func(task='倒水茶水晚上',date=date))
    except TaskInfo as e:
        return e
    return 'success'

def evening()->str:
    """晚间休息的任务

    Returns:
        str: 系统消息
    """
    date = Date().date
    try:
        check_(create_func(task='整理家里和猫猫',date=date))
        check_(create_func(task='晚上洗漱,刷牙,护肤',date=date))
        check_(create_func(task='锻炼',date=date))
        check_(create_func(task='睡觉',date=date))
    except TaskInfo as e:
        return e

    return 'success'

def rest()->str:
    date = Date().date
    try:
        check_(create_func(task='吃早饭',date=date))
        check_(create_func(task='洗漱',date=date))
        check_(create_func(task='锻炼',date=date))
        check_(create_func(task='睡觉',date=date))
    except TaskInfo as e:
        return e
    return "success"

from chinese_calendar import is_workday
from datetime import datetime

def receive()->str:
    """领取一个任务(普通模式)

    Returns:
        str: 任务信息
    """
    time = Date().time
    date = Date().date
    DEBUG = Setting().debug
    date_obj = datetime.strptime(date, '%Y-%m-%d').date()

    if is_workday(date_obj):
        time = datetime.strptime(f"{date} {time}", "%Y-%m-%d %H:%M:%S")
        # Create datetime objects once to avoid repetition
        time_8_50 = datetime.strptime(date + " 8:50:00", "%Y-%m-%d %H:%M:%S")
        time_10_00 = datetime.strptime(date + " 10:00:00", "%Y-%m-%d %H:%M:%S") 
        time_18_00 = datetime.strptime(date + " 18:00:00", "%Y-%m-%d %H:%M:%S")
        time_19_00 = datetime.strptime(date + " 19:00:00", "%Y-%m-%d %H:%M:%S")
        time_23_00 = datetime.strptime(date + " 23:00:00", "%Y-%m-%d %H:%M:%S")
        
        if time < time_8_50:
            return morning()

        elif time_8_50 < time <= time_10_00:
            return start_work(debug=DEBUG)

        elif time_10_00 <= time < time_18_00:
            return tasks()

        elif time_18_00 <= time < time_19_00:
            return finish_work()

        elif time_19_00 <= time < time_23_00:
            return evening()

        return 'success'
    else:
        rest()


def complete():
    """完成当前任务
    """
    date = Date().date
    task = receive()
    push_task(task=task,date=date)


#TODO 强化型健身  1周一次 在突破技能   5000米 + 负重 + 拳击


""" 动作函数 """
from .kanbantools import KanBanManager
from .appiotools import APPIO
from reallife import KANBAN_PATH,WORK_CANVAS_PATH,Date
from .utils import status

date_c = Date()
date = date_c.date

@status(task="同步体重",date=date,run_only=True)
def sync_weight():
    """同步体重
    """
    kanb = KanBanManager(kanban_path=KANBAN_PATH,
                         pathlib=WORK_CANVAS_PATH)
    kanb.sync_weight(date)

@status(task="同步预备池",date=date,run_only=True)
def sync_ready_pool()->str:
    """同步预备池
    """
    kanb = KanBanManager(kanban_path=KANBAN_PATH,
                         pathlib=WORK_CANVAS_PATH)
    kanb.sync_ready()

@status(task="同步就绪池",date=date,run_only=True)
def sync_order_pool()->str:
    """同步就绪池
    """
    kanb = KanBanManager(kanban_path=KANBAN_PATH,
                         pathlib=WORK_CANVAS_PATH)
    kanb.sync_order()

@status(task="同步执行池",date=date,run_only=True)
def sync_run_pool()->str:
    """同步执行池
    """
    kanb = KanBanManager(kanban_path=KANBAN_PATH,
                         pathlib=WORK_CANVAS_PATH)
    kanb.sync_run()

@status(task="同步日历",date=date,run_only=True)
def sync_calulate()->str:
    """同步日历
    """
    appio = APPIO()
    appio.sync_calulate()


@status(task="同步备忘录",date=date,run_only=True)
def sync_note()->str:
    """同步备忘录
    """
    appio = APPIO()
    appio.sync_notes()

@status(task="同步备忘录夜",date=date,run_only=True)
def sync_note_night()->str:
    """同步备忘录夜
    """
    appio = APPIO()
    appio.sync_notes()

@status(task="回收未完成的任务",date=date,run_only=True)
def recycle_tasks()->str:
    """回收未完成的任务
    """
    kanb = KanBanManager(kanban_path=KANBAN_PATH,
                         pathlib=WORK_CANVAS_PATH)
    kanb.sync_run2order()

@status(task="收集资讯",date=date,run_only=True)
def sync_news()->str:
    """收集资讯
    """
    appio = APPIO()
    appio.sync_news(date)




prompt = """
你是一个计划构筑师,根据我的任务帮我拆分编写工作流程与计划, 使用mermaid 的方式构建.

注意: 相对简明, mermaid格式简单,清晰

---
任务:
{任务}

"""



import requests
from bs4 import BeautifulSoup
import os
# 配置相关
from llama_index.llms.openai import OpenAI

import requests
from bs4 import BeautifulSoup
from datetime import datetime

from functools import lru_cache


# 初始化 LLM 和 Embedding 模型
llm = OpenAI(
    model="gpt-4.1-2025-04-14",
    api_key=os.getenv("BIANXIE_API_KEY"),
    api_base='https://api.bianxieai.com/v1',
    temperature=0.1,
)
obsidian = "/Users/zhaoxuefeng/GitHub/obsidian"

def main(task:str):
    return llm.complete(prompt.format(任务=task))



if __name__ == "__main__":
    import sys
    multi_line_input = sys.stdin.read()
    print(main(multi_line_input))


""" 初始化 设定值"""

from llmada import BianXieAdapter
from .utils import Date, Setting, load_config
# Load configuration
config = load_config()
KANBAN_PATH = config.get('KANBAN_PATH', '')
WORK_CANVAS_PATH = config.get('WORK_CANVAS_PATH', [])

setting = Setting()
llm = BianXieAdapter()
llm.set_model("gemini-2.5-flash-preview-04-17-nothinking")
setting.llm = llm
setting.debug = False

# Date(date = '2025-01-01',time = '14:40:10')
Date()




####

""" 系统工具箱 """

from datetime import datetime
import subprocess
import shlex
import os
import re
from typing import Optional
import functools
import requests

import importlib.resources
import yaml

# Load configuration from YAML file
def load_config():
    with importlib.resources.open_text('reallife', 'config.yaml') as f:
        return yaml.safe_load(f)
    # with open('reallife/config.yaml', 'r', encoding='utf-8') as f:
    #     return yaml.safe_load(f)

class TaskInfo(Exception):
    """任务的抛出机制

    Args:
        Exception (_type_): 抛出
    """
    def __init__(self, message):
        self.message = message
        super().__init__(self.message)

def check_(func):
    """检查信息的封装

    Args:
        func (object): 信息方法

    Raises:
        TaskInfo: 遇到信息抛出消息等待被捕捉
    """
    result = func()
    if result:
        raise TaskInfo(result)

def check_action_(func,debug=True):
    """一个执行动作的配装

    Args:
        func (object): 动作函数
        debug (bool, optional): 是否debug模式. Defaults to True.
    """
    result = func()
    if result and debug:
        print(func.__name__)

def git_commit_something(repo:str,commit:str):
    """提交git commit 

    Args:
        repo (str): 仓库
        commit (str): 提交信息
    """
    os.system(f"cd /Users/zhaoxuefeng/GitHub/{repo};git add .; \
              git commit -m '{commit}' --allow-empty")


def failed_safe():
    # 安全开启 放弃
    '''
    查询session状态
    如果 当前session 状态是 session
        放弃session
    否则
        pass
    结束
    '''
    run_shortcut(shortcut_name="安全停止")


def task_with_time(task_name:str,time:int=1):
    """计时任务

    Args:
        task_name (str): 任务名
        time (int, optional): 费用. Defaults to 1.
    """
    run_shortcut(shortcut_name="执行计时任务",params=f"{task_name}${time}")
    display_dialog("计时结束", "需要结束计时任务吗?",button_text="结束",button_cancel=False)



class Date:
    """日期, 来控制系统时间

    Returns:
        object: 单例模式
    """
    _instance = None
    def __new__(cls, *args, **kwargs):
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    def __init__(self,date:str = None,time:str = None):
        # 只有第一次初始化时设置值，后续的初始化调用不会更改实例的值
        if not hasattr(self, 'initialized'):
            self.initialized = True
            self.date = date or datetime.today().strftime("%Y-%m-%d")
            self.time = time or datetime.today().strftime("%H:%M:%S")


class Setting:
    """日期, 来控制系统时间

    Returns:
        object: 单例模式
    """
    _instance = None
    def __new__(cls, *args, **kwargs):
        if cls._instance is None:
            cls._instance = super().__new__(cls)
        return cls._instance

    def __init__(self):
        # 只有第一次初始化时设置值，后续的初始化调用不会更改实例的值
        if not hasattr(self, 'initialized'):
            self.initialized = True
            self.llm = None
            self.debug = None





def push_task(task:str, date:str):
    """推任务上去

    Args:
        task (str): 任务名
        date (str): 日期
    """
    requests.post(
        f"http://101.201.244.227:8000/action/{task}/{date}/execute",timeout=20)
    return 'success'

def reset_task(task:str,date:str)->str:
    """重置任务

    Args:
        task (str): 任务名
        date (str): 日期

    Returns:
        str: 系统消息
    """
    requests.post(
        f"http://101.201.244.227:8000/action/{task}/{date}/reset",timeout=20)
    return 'success'

def reset_tasks(date:str):
    """重置多任务

    Args:
        date (str): 日期
    """
    for task in ['同步进程池','同步预备池','同步就绪池','同步执行池',]:
        reset_task(task,date)


def create_func(task:str,date:str)->object:
    """创建特定对象

    Args:
        task (str): 任务
        date (str, optional): 日期. Defaults to date.

    Returns:
        object: 状态函数
    """
    @status(task=task,date=date)
    def func_():
        """提醒

        Returns:
            str: 任务内容
        """
        return task
    return func_



from llmada import BianXieAdapter

class Meeting():
    def __init__(self,system:str = None, members = None):
        self.system = system or "让我们现在模拟一个开会讨论的过程, 我们会有一个主题以及四个与会者, 我们会通过不断的讨论对齐我们的想法,最终形成一个相对全面完整的共识. 以聊天的形式进行"
        self.aiTeam = BianXieAdapter()
        self.history = [{"role":"system","content":self.system}]
        self.members = members or {
                        "gpt-4o":"gpt-4o",
                        "gemini":"gemini-2.5-pro-exp-03-25",
                        "gpt-4.1":"gpt-4.1",
                        }
        
    def guide(self,user_message:str,model:str = "",):
        self.history.append({'role': 'user', 'content': user_message})
        self.aiTeam.set_model(model)
        result = self.aiTeam.chat(self.history)
        self.history.append({'role':'assistant','content':f'发言者 {model}:\n' + result})
        return result
    
    def start(self):
        for model_id in ["gpt-4o","gemini","gpt-4.1","gpt-4o","gemini","gpt-4.1","gpt-4o","gemini","gpt-4.1",
                          "gpt-4o","gemini","gpt-4.1"]:
            model = self.members.get(model_id)
            print("model: ",model)
            user_message = input()
            result = self.guide(user_message,model)
            print(f'发言者 {model}:\n' + result)

            